int mouse;

/*------------------------------ start_mouse () -----------------------*\
| This function initializes the mouse driver by using service 0 of      |
| interrupt 33H then it enables the cursor by calling enable_cursor()   |
|           IN   : NONE                                                 |
|           OUT  : NONE                                                 |
\*---------------------------------------------------------------------*/
start_mouse()
{
union REGS r;
r.x.ax=0;               /* service 0 in AX */
int86(0x33,&r,&r);      /* call interrupt */
if(r.x.ax==0) mouse=0;  /* mouse status returned in AX */
else mouse=1;
enable_cursor();        /* enable cursor on screen */
}

/*---------------------- disable_cursor () ----------------------------*\
| This function disables the mouse cursor by using sevice 2 of interrupt| 
| 33H and this function is used when changing any part of the screen.   |
|             IN  : NONE                                                |
|             OUT : NONE                                                |
\*---------------------------------------------------------------------*/      
disable_cursor()
{
union REGS r;
if(mouse==1)                    /* if mouse was detected... */
	{
	r.x.ax=2;               /* then call service 2 */
	int86(0x33,&r,&r);      /* of interrupt 33H */
	}
}

/*---------------------- enable_cursor () -----------------------------*\
| This function enables the mouse cursor by using sevice 1 of interrupt |
| 33H.                                                                  |
|             IN  : NONE                                                |
|             OUT : NONE                                                |
\*---------------------------------------------------------------------*/      
enable_cursor()
{
union REGS r;
if(mouse==1)                    /* if mouse was detected */
	{
	r.x.ax=1;               /* call seivice 1 */
	int86(0x33,&r,&r);      /* of interrupt 33H */
	}
}

/*---------------------- get_mousex () --------------------------------*\
| This function returns the x-coordinate of the mouse cursor. It calls  |
| service 3 in interrupt 33H.                                           |
|             IN  : NONE                                                |
|             OUT : NONE                                                |
\*---------------------------------------------------------------------*/
get_mousex()
{
union REGS r;
if(mouse==1)
	{
	r.x.ax=3;               /* call service 3 */
	int86(0x33,&r,&r);      /* of interrupt 33 H */
	return(r.x.cx);         /* x-coordinate is in CX */
	}
}

/*---------------------- get_mousey () --------------------------------*\
| This function returns the x-coordinate of the mouse cursor. It calls  |
| service 3 in interrupt 33H.                                           |
|             IN  : NONE                                                |
|             OUT : NONE                                                |
\*---------------------------------------------------------------------*/
get_mousey()
{
union REGS r;
if(mouse==1)
	{
	r.x.ax=3;               /* call service 3 */
	int86(0x33,&r,&r);      /* of interrupt 33 H */
	return(r.x.dx);         /* y-coordinate is in DX */
	}
}

/*---------------------- check_buttons () -----------------------------*\
| This function checks the mouse buttons status using service 3 of int  |
| 33H and pokes character codes to the keyboard buffer according to the |
| button pressed. For example if the right button is pressed it pokes   |
| <ENTER> and if the middle is pressed it pokes <F1> and so on...       |
| Poking a character into the keyboard buffer is performed by calling   |
| service 5, Int 16H.                                                   |
| In this function I wrote a separate 'if' for each button and the same |
| function can be done with a code of half of what I wrote using switch |
| but I do this for the speed as this function is called always from all| 
| functions                                                             |
|             IN  :  NONE                                               |
|             OUT :  Indirect output in the form of characters in the   | 
|                    keyboard buffer.                                   |
|                    Also returns 1 if either button has been pressed.  |
\*---------------------------------------------------------------------*/
check_buttons()
{
int returnvalue=0,x,y;
union REGS r;
if(mouse==1)                    /* if mouse was detected */
	{
	r.x.ax=3;               /* call service 3 */
	int86(0x33,&r,&r);      /* of interrupt 33 H */
	if(r.x.bx==1)           /* if left button was pressed... */
		{               /* then push ENTER into keayboard buffer */
		clear_buffer();
		r.h.ch=0x81;    
		r.h.cl=0x0;
		r.h.ah=5;
		int86(0x16,&r,&r);
		returnvalue=1;
		}
	if(r.x.bx==2)           /* if right button was pressed... */
		{               /* insert ESC into keyboard buffer */
		clear_buffer();
		r.h.ch=1;       /* Scan code of ESC is 1 */
		r.h.cl=0x1b;    /* ASCII code of ESC is 1bH = 27 */
		r.h.ah=5;               /* call service 5 */
		int86(0x16,&r,&r);      /* of interrupt 16 H */
		returnvalue=1;
		}
	if(r.x.bx==4)           /* if middl button was pressed... */
		{               /* insert F! into keyboard buffer */
		clear_buffer();
		r.h.ch=0x3b;    /* SCAN code of F1 */
		r.h.cl=0;       /* ASCII code of F1 */
		r.h.ah=5;               /* call service 5 */
		int86(0x16,&r,&r);      /* of interrupt 16 H */
		returnvalue=1;
		}
}
return(returnvalue);
}

